#!/usr/bin/python
import regress
import unicorn as U

class WrongConditionalPath(regress.RegressTest):
    def test_eflags(self):
        # 0:    4d 31 f6                 xor    r14, r14
        # 3:    45 85 f6                 test   r14d, r14d
        # 6:    75 fe                    jne    0x6
        # 8:    f4                       hlt
        CODE = 'M1\xf6E\x85\xf6u\xfe\xf4'

        uc = U.Uc(U.UC_ARCH_X86, U.UC_MODE_64)
        uc.reg_write(U.x86_const.UC_X86_REG_RIP, 0x6000b0)
        uc.reg_write(U.x86_const.UC_X86_REG_EFLAGS, 0x246)

        uc.mem_map(0x600000, 0x1000)
        uc.mem_write(0x6000b0, CODE)

        uc.emu_start(0x6000b0 + 6, 0, count=1)

        # Here's the original execution trace for this on qemu-user.
        #
        # $ SC='xor r14,r14; test r14d, r14d; jne $; hlt'
        # $ asm --context amd64 --format elf $SC > example
        # $ qemu-x86_64-static -d cpu,in_asm -singlestep ./test \
        #   | grep -E 'RFL|^0x'
        # 0x00000000006000b0:  xor    %r14,%r14
        # RIP=00000000006000b0 RFL=00000202 [-------] CPL=3 II=0 A20=1 SMM=0 HLT=0
        # 0x00000000006000b3:  test   %r14d,%r14d
        # RIP=00000000006000b3 RFL=00000246 [---Z-P-] CPL=3 II=0 A20=1 SMM=0 HLT=0
        # 0x00000000006000b6:  jne    0x6000b6
        # RIP=00000000006000b6 RFL=00000246 [---Z-P-] CPL=3 II=0 A20=1 SMM=0 HLT=0
        # 0x00000000006000b8:  hlt    
        # RIP=00000000006000b8 RFL=00000246 [---Z-P-] CPL=3 II=0 A20=1 SMM=0 HLT=0
        self.assertEqual(0x6000b0 + 8, uc.reg_read(U.x86_const.UC_X86_REG_RIP))

if __name__ == '__main__':
    regress.main()



from unicorn import *
from unicorn.mips_const import *



# .text:00416CB0                 cfc1    $v1, FCSR
shellcode = [0x44, 0x43, 0xF8, 0x00]
base = 0x416CB0

uc = Uc(UC_ARCH_MIPS, UC_MODE_MIPS32 + UC_MODE_BIG_ENDIAN)
uc.mem_map(0x416000, 0x1000)
uc.mem_write(base, bytes(shellcode))
uc.emu_start(base, base + len(shellcode))


from unicorn import *
from unicorn.x86_const import *

count = 0

def cb(uc, addr, sz, data):
    global count
    count += 1
    print(f"addr: {hex(addr)} count: {count}")
    if count == 5:
        uc.emu_stop()
    else:
        uc.reg_write(UC_X86_REG_RIP, 0x2000)

mu = Uc(UC_ARCH_X86, UC_MODE_64)

mu.mem_map(0x1000, 0x4000)
mu.mem_write(0x1000, b"\x90" * 5)
mu.mem_write(0x2000, b"\x90" * 5)
mu.hook_add(UC_HOOK_CODE, cb)
mu.emu_start(0x1000, 0x2000+1, 0, 0)



from unicorn import *
from unicorn.arm_const import *

# ADD R0, R10, R0;
# B L0;
# L0:
#   ADD R0, R10, R0; <--- we stop at here, the first instruction of the next TB.

code = b'\x00\x00\x8a\xe0\xff\xff\xff\xea\x00\x00\x8a\xe0'
address = 0x1000

mu = Uc(UC_ARCH_ARM, UC_MODE_ARM)
mu.mem_map(address, 0x1000)
mu.mem_write(address, code)
mu.emu_start(address, address + len(code) - 4)